<?php
namespace AkTest\Task;

use Ak\Orm\Entity\EntityInterface;
use Catequil\ORM\ConfigProvider as CatequilOrmConfigProvider;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Tools\Console\ConsoleRunner;
use Psr\Container\ContainerInterface;
use Symfony\Component\Console\Application;
use Symfony\Component\Console\Input\StringInput;
use Symfony\Component\Console\Output\NullOutput;

/**
 * Trait ORMTestTrait
 *
 * @package Ak\Test\Orm
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://github.com/hipnaba/tiara-openid
 *
 * @property array $providers
 *
 * @method ContainerInterface getContainer(array $config = [])
 */
trait ORMTestTrait
{
    /**
     * Inicijalizira ORM sustav.
     */
    protected function setUpOrm(): void
    {
        if (!in_array(CatequilOrmConfigProvider::class, $this->providers)) {
            array_unshift($this->providers, CatequilOrmConfigProvider::class);
        }
    }

    /**
     * Kreira schemu definiranu u konfiguraciji.
     *
     * @param EntityManagerInterface|null $em
     */
    protected function createSchema(EntityManagerInterface $em = null): void
    {
        $this->setUpOrm();
        $this->dropSchema($em);
        $this->runCliCommand('orm:schema-tool:create', $em);
    }

    protected function dropSchema(EntityManagerInterface $em = null): void
    {
        $this->setUpOrm();
        $this->runCliCommand('orm:schema-tool:drop --force', $em);
    }/** @noinspection PhpDocMissingThrowsInspection */

    /**
     * @param string $command
     * @param EntityManagerInterface|null $em
     */
    private function runCliCommand(string $command, EntityManagerInterface $em = null): void
    {
        $cli = $this->getCliApplication($em);
        /** @noinspection PhpUnhandledExceptionInspection */
        $cli->run(new StringInput($command), new NullOutput());
    }

    /**
     * @param EntityManagerInterface|null $em
     * @return Application
     */
    private function getCliApplication(EntityManagerInterface $em = null): Application
    {
        if (null === $em) {
            $em = $this->getEntityManager();
        }

        $helperSet = ConsoleRunner::createHelperSet($em);
        $cli = ConsoleRunner::createApplication($helperSet, []);
        $cli->setAutoExit(false);
        $cli->setCatchExceptions(false);

        return $cli;
    }

    /**
     * @return EntityManagerInterface|EntityManager
     */
    protected function getEntityManager(): EntityManagerInterface
    {
        $container = $this->getContainer();
        return $container->get(EntityManagerInterface::class);
    }

    /**
     * @return Connection
     */
    protected function getConnection(): Connection
    {
        $em = $this->getEntityManager();
        return $em->getConnection();
    }/** @noinspection PhpDocMissingThrowsInspection */

    /**
     * @param string $entityClass
     * @param int $id
     * @return EntityInterface|object
     */
    protected function findEntity(string $entityClass, int $id): EntityInterface
    {
        $em = $this->getEntityManager();
        /** @noinspection PhpUnhandledExceptionInspection */
        return $em->find($entityClass, $id);
    }/** @noinspection PhpDocMissingThrowsInspection */

    /**
     * @param EntityInterface $entity
     */
    protected function persistEntity(EntityInterface $entity): void
    {
        $em = $this->getEntityManager();
        /** @noinspection PhpUnhandledExceptionInspection */
        $em->persist($entity);
        /** @noinspection PhpUnhandledExceptionInspection */
        $em->flush();
    }
}